import xml_sax, xml.sax

class Error(Exception):
    pass

class StopSaxProcessing(Exception):
    pass

class DescriptionContentHandler(xml.sax.handler.ContentHandler):
    """SAX handler which extracts /migration-dump/description/text()"""
    def __init__(self):
        self.description = ""
        self.inMigrationDumpNode = False
        self.inDescriptionNode = False

    def startElement(self, name, attrs):
        if name == "migration-dump":
            self.inMigrationDumpNode = True
            return

        if self.inMigrationDumpNode and name == "description":
            self.inDescriptionNode = True
            return

        if self.inDescriptionNode:
            raise Error("Child node in <description> node occured")

        if self.inMigrationDumpNode:
            raise StopSaxProcessing()

        raise Error("Top-level node is not <migration-dump>")

    def characters(self, content):
        if self.inDescriptionNode:
            self.description += content

    def endElement(self, name):
        if name == "description":
            raise StopSaxProcessing()


def textNode(writer, nodeName, content):
    writer.startElement(nodeName, {})
    writer.characters(content)
    writer.endElement(nodeName)


### Extracting objects structure from dump

class Client(object):
    def __init__(self, name, contact, guid = None):
        self.name = name
        self.contact = contact
        self.guid = guid
        self.ips = []
        self.domains = []

    def serialize(self, writer):
        writer.startElement("client", {})
        textNode(writer, "name", self.name)
        if self.contact:
            textNode(writer, "contact", self.contact)
        for i in self.ips:
            i.serialize(writer)
        for d in self.domains:
            d.serialize(writer)
        writer.endElement("client")


class Ip(object):
    def __init__(self):
        self.type_ = ""
        self.address = ""

    def serialize(self, writer):
        writer.startElement("ip", {})
        textNode(writer, "ip-type", self.type_)
        textNode(writer, "ip-address", self.address)
        writer.endElement("ip")

class SystemIp(object):
    def __init__(self):
        self.type_ = ""
        self.address = ""
        self.netmask = ""
        self.interface = ""

    def serialize(self, writer):
        writer.startElement("ip", {})
        textNode(writer, "ip-type", self.type_)
        textNode(writer, "ip-address", self.address)
        textNode(writer, "ip-netmask", self.netmask)
        textNode(writer, "ip-interface", self.interface)
        writer.endElement("ip")


class Domain(object):
    def __init__(self, name, guid = None):
        self.name = name
        self.guid = guid
        self.ip = None
        self.dbServers = []

    def normalizeDbServers(self):
        dbs = self.dbServers
        dbs.sort()

        self.dbServers = []
        if len(dbs) > 0:
            currDbs = dbs.pop(-1)
            self.dbServers.append(currDbs)
        for i in xrange(0, len(dbs)):
            if currDbs != dbs[i]:
                currDbs = dbs[i]
                self.dbServers.append(currDbs)

    def serialize(self, writer):
        writer.startElement("domain", {})
        textNode(writer, "name", self.name)
        if self.ip:
            self.ip.serialize(writer)
        for d in self.dbServers:
            d.serialize(writer)
        writer.endElement("domain")


class DbServer(object):
    def __init__(self, serverType, serverDefault):
        self.serverType = serverType
        self.serverDefault = serverDefault
        self.host = ""
        self.port = ""
        self.admin = None

    def __cmp__(self, o):
        if self.serverType != o.serverType: return cmp(self.serverType, o.serverType)
        if self.host != o.host: return cmp(self.host, o.host)
        return cmp(self.port, o.port)

    def serialize(self, writer):
        writer.startElement("db-server", {})
        textNode(writer, "host", self.host)
        textNode(writer, "port", self.port)
        textNode(writer, "type", self.serverType)
        if self.admin:
            self.admin.serialize(writer)
        else:
            textNode(writer, "admin", "")
            textNode(writer, "password", "")
        if self.serverDefault == "true":
            textNode(writer, "default", "")
        writer.endElement("db-server")


class DbServerAdmin(object):
    def __init__(self, name):
        self.name = name
        self.password = None

    def serialize(self, writer):
        textNode(writer, "admin", self.name)
        self.password.serialize(writer)


class Password(object):
    def __init__(self):
        self.value = ""

    def serialize(self, writer):
        textNode(writer, "password", self.value)


class InfoContentHandler(xml.sax.handler.ContentHandler):
    """SAX handler which extracts structure of objects from dump"""

    def __init__(self):
        self._nodesContext = ['']

        self.description = ""
        self.dumpVersion = ""
        self.clients = []
        self.domains = []
        self.dbServers = []

        self.ssoEnabled = None
        self.ssoAdminUid = ""
        self.ssoServer = ""
        self.ssoRelay = ""
        self.ssoApplicationId = ""

        # indicates if there were server-wide settings
        # TODO: introduce class Server for such purpose
        self.server = False

        self.systemIps = []
        self.defaultIp = None
        self.hostname = None
        self.adminPassword = None

        self.hasMailSettings = False
        self.hasDnsZone = False
        self.hasCertificate = False
        self.hasCustomButton = False
        self.hasDomainTemplate = False
        self.hasClientTemplate = False
        self.hasApplicationVault = False

        self.currentDbServer = None
        self.currentClient = None
        self.currentDomain = None
        self.currentIp = None

        self.cids_in_content = ("sapp-scripts", "sapp-cgi-bin", "sapp-htdocs")
        self.currentCid = ""

        self.cids = {}

    def _parent(self): return self._nodesContext[-1]
    def _push(self, name): self._nodesContext.append(name)
    def _pop(self): self._nodesContext.pop(-1)

    def _add_cid(self, cid):
        if self.currentDomain:   key = "domain_" + self.currentDomain.name
        elif self.currentClient: key = "client_" + self.currentClient.name
        else:                    key = "server"

        if key in self.cids:
            self.cids[key].append(cid)
        else:
            self.cids[key] = [cid]

    def startElement(self, name, attrs):
        if name == "client":
            self.currentClient = Client(attrs.get("name"), attrs.get("contact"), attrs.get("guid"))
            self.clients.append(self.currentClient)
        elif name == "server":
            self.server = True
        elif name == "system-ip":
            self.systemIps.append(SystemIp())
            self.currentIp = self.systemIps[-1]
        elif name == "ip":
            if self._parent() == "ip_pool":
                self.currentClient.ips.append(Ip())
                self.currentIp = self.currentClient.ips[-1]
            elif self._parent() == "domain":
                self.currentDomain.ip = Ip()
                self.currentIp = self.currentDomain.ip
        elif name == "default-ip" and self._parent() == "server":
            self.defaultIp = ""
        elif name == "hostname" and self._parent() == "server":
            self.hostname = ""

        elif name == "mail-settings":
            self.hasMailSettings = True
        elif name == "dns-settings":
            self.hasDnsZone = True
        elif name == "certificate" and self._parent() == "server":
            self.hasCertificate = True
        elif name == "custom-button" and self._parent() == "server":
            self.hasCustomButton = True
        elif name == "domain-template" and self._parent() == "server":
            self.hasDomainTemplate = True
        elif name == "client-template" and self._parent() == "server":
            self.hasClientTemplate = True
        elif name == "application-vault" and self._parent() == "server":
            self.hasApplicationVault = True

        elif name == "sso-settings" and self._parent() == "server":
            self.ssoEnabled = attrs.get("enabled")

        elif name == "domain":
            self.currentDomain = Domain(attrs.get("name"), attrs.get("guid"))
            if self._parent() == "migration-dump":
                self.domains.append(self.currentDomain)
            else:
                self.currentClient.domains.append(self.currentDomain)
        elif name == "db-server":
            self.currentDbServer = DbServer(attrs.get("type"), attrs.get("default", "false"))
            if self._parent() == "server":
                self.dbServers.append(self.currentDbServer)
            elif self._parent() == "database":
                self.currentDomain.dbServers.append(self.currentDbServer)
        elif name == "admin":
            self.currentDbServer.admin = DbServerAdmin(attrs.get("name"))
        elif name == "password":
            if self._parent() == "admin":
                self.currentDbServer.admin.password = Password()
        elif name == "admin-password":
            self.adminPassword = ""
        elif name == "migration-dump" and attrs.has_key('dump-version'):
            self.dumpVersion = attrs.get("dump-version")

        for attr, cid in attrs.items():
            if attr[:3] == "cid":
               self._add_cid(cid)

        self._push(name)

    def characters(self, content):
        if self._parent() == 'description':
            self.description += content

        elif self._parent() == 'host':
            self.currentDbServer.host += content

        elif self._parent() == 'port':
            self.currentDbServer.port += content

        elif self._parent() == 'password' and self._nodesContext[-2] == "admin":
            self.currentDbServer.admin.password.value += content

        elif self._parent() == 'ip-type':
            self.currentIp.type_ += content

        elif self._parent() == 'ip-address':
            self.currentIp.address += content

        elif self._parent() == 'default-ip':
            self.defaultIp += content

        elif self._parent() == 'hostname':
            self.hostname += content

        elif self._parent() == 'admin-password':
            self.adminPassword += content

        elif self._parent() == 'ip-netmask':
            self.currentIp.netmask += content

        elif self._parent() == 'ip-interface':
            self.currentIp.interface += content

        elif self._parent() == 'sso-admin-uid':
            self.ssoAdminUid += content

        elif self._parent() == 'sso-server':
            self.ssoServer += content

        elif self._parent() == 'sso-relay':
            self.ssoRelay += content

        elif self._parent() == 'sso-application-id':
            self.ssoApplicationId += content

        elif self._parent() in self.cids_in_content:
            self.currentCid += content

    def endElement(self, name):
        if name == "client":
            self.currentClient = None
        elif name == "domain":
            self.currentDomain.normalizeDbServers()
            self.currentDomain = None
        elif name == "db-server":
            self.currentDbServer = None
        elif name == "ip":
            if self._parent() == "ip_pool" or self._parent() == "domain":
                self.currentIp = None
        elif name == "system-ip":
            self.currentIp = None

        elif name in self.cids_in_content:
            self._add_cid(self.currentCid)
            self.currentCid = ""

        self._pop()

    def serialize(self, writer):
        writer.startDocument()
        self.serializeContent(writer)
        writer.endDocument()

    def serializeContent(self, writer, elementName = "awaiting-objects"):
        writer.startElement(elementName, {})
        textNode(writer, "description", self.description)
        textNode(writer, "dump-version", self.dumpVersion)
        for c in self.clients:
            c.serialize(writer)
        for d in self.domains:
            d.serialize(writer)
        # Output server element only if there are server-related settings
        if self.server:
            writer.startElement("server", {})
            for s in self.dbServers:
                s.serialize(writer)
            for i in self.systemIps:
                i.serialize(writer)
            if self.defaultIp:
                textNode(writer, "default-ip", self.defaultIp)
            if self.hostname:
                textNode(writer, "hostname", self.hostname)
            if self.adminPassword:
                textNode(writer, "password", self.adminPassword)

            if self.ssoEnabled != None:
                textNode(writer, "sso-enabled", self.ssoEnabled)
                textNode(writer, "sso-admin-uid", self.ssoAdminUid)
                textNode(writer, "sso-server", self.ssoServer)
                textNode(writer, "sso-relay", self.ssoRelay)
                textNode(writer, "sso-application-id", self.ssoApplicationId)

            if self.hasMailSettings:
                textNode(writer, "mail-settings", "")
            if self.hasDnsZone:
                textNode(writer, "dns-settings", "")
            if self.hasCertificate:
                textNode(writer, "certificate", "")
            if self.hasCustomButton:
                textNode(writer, "custom-button", "")
            if self.hasDomainTemplate:
                textNode(writer, "domain-template", "")
            if self.hasClientTemplate:
                textNode(writer, "client-template", "")
            if self.hasApplicationVault:
                textNode(writer, "application-vault", "")

            writer.endElement("server")

        writer.endElement(elementName)

    #generateDS compatibilty
    def export(self, outfile, level, name_):
        writer = xml_sax.myXMLGenerator(outfile, "UTF-8")
        self.serializeContent(writer, name_)


def readDescription(fp):
    i = DescriptionContentHandler()
    try:
        xml.sax.parse(fp, i)
    except StopSaxProcessing:
        pass

    return i.description

def readInfo(fp):
    i = InfoContentHandler()
    try:
        xml.sax.parse(fp, i)
    except StopSaxProcessing:
        pass

    return i

if __name__ == '__main__':
    import sys
    readInfo(file(sys.argv[1])).serialize(xml_sax.myXMLGenerator(sys.stdout, "UTF-8"))

# Local Variables:
# mode: python
# indent-tabs-mode: nil
# tab-width: 4
# End:
