<?php

require_once(dirname(__FILE__) . '/helpers/base.php');

/**
 * Locale import utility
 *
 */
class Lkit_Application_Import extends Lkit_Application_Abstract
{

    private $_inputDir;

    /**
     * Run application
     *
     */
    public function run()
    {
        $this->_log("Locale importer.");

        $this->_initEnvironment();
        $this->_parseOptions();
        $this->_validateLocale();
        $this->_importLocale();
    }

    /**
     * Parse command line options
     *
     */
    private function _parseOptions()
    {
        $options = new Zend_Console_Getopt(
            array(
                'dir|d=s' => 'Locale source directory (for example, locale-en-US).',
            )
        );

        $arguments = $_SERVER["argv"];

        if ((count($arguments) < 2) || !$options->dir) {
            echo $options->getUsageMessage();
            exit(1);
        }

        $this->_inputDir = $options->dir;

        if (!is_dir($this->_inputDir)) {
            $this->_fatalError("Specified directory $this->_inputDir was not found.");
        }

        $this->_log("Locale source directory: $this->_inputDir");
    }

    /**
     * Validate locale source directory content
     *
     */
    private function _validateLocale()
    {
        if (!file_exists("$this->_inputDir/info.xml")) {
            $this->_fatalError("Language pack description file $this->_inputDir/info.xml is not found.");
        }

        $xml = $this->_getLocaleDescriptionXml();
        $localeCode = $xml->code;

        if (!preg_match('/^[a-z]{2}-[A-Z]{2}$/', $localeCode)) {
            $this->_fatalError("Wrong locale code is specified in the file $this->_inputDir/info.xml, it must be in the ll-CC format.");
        }

        $platform = $xml->platform;

        if (($this->_isOsUnix() && ('unix' != $xml->platform))
            || ($this->_isOsWindow() && ('windows' != $xml->platform))
        ) {
            $this->_fatalError("The language pack is designed for different platform.");
        }

        $this->_validateRequiredFiles();

        $this->_log("Language pack is successfully validated.");
    }

    /**
     * Check if all required files are present
     *
     */
    private function _validateRequiredFiles()
    {
        $requiredFiles = array('panel/common_messages.php', 'panel/conhelp.php',
            'panel/messages.php', 'siteeditor/locale.xml', 'smb/locale.php');

        if ($this->_isOsWindow()) {
            $requiredFiles[] = 'panel/cli_messages.php';
        }

        foreach ($requiredFiles as $requiredFile) {
            if (!file_exists("$this->_inputDir/$requiredFile")) {
                $this->_fatalError("Required file '$requiredFile' is not found.");
            }

            // validate syntax in PHP files
            if ('.php' === substr($requiredFile, -4, 4)) {
                $this->_exec("$this->_phpBinary -l \"$this->_inputDir/$requiredFile\"");
            }
        }
    }

    /**
     * Import locale
     *
     */
    private function _importLocale()
    {
        $this->_localeCode = $this->_getLocaleDescriptionXml()->code;
        $this->_log("Locale code: $this->_localeCode");

        $this->_importSmbLocale();
        $this->_log("Step 1 out of 3 is done.");
        $this->_importPleskLocale();
        $this->_log("Step 2 out of 3 is done.");
        $this->_importSiteEditorLocale();
        $this->_log("Step 3 out of 3 is done.");

        $this->_log("$this->_localeCode language pack is successfully imported.");
    }

    /**
     * Import SMB locale
     *
     */
    private function _importSmbLocale()
    {
        $localeFile = "$this->_inputDir/smb/locale.php";
        $outputDir = "$this->_productSmbDir/resources/languages/";
        $converter = "$this->_productSmbDir/utils/locale/plesk2smb.php";
        $this->_exec("$this->_phpBinary \"$converter\" -f \"$localeFile\" -o \"$outputDir\" -l $this->_localeCode");
    }

    /**
     * Import Plesk locale
     *
     */
    private function _importPleskLocale()
    {
        $inputDir = "$this->_inputDir/panel/";
        $outputDir = "$this->_productBaseDir/admin/plib/locales/$this->_localeCode/";

        if (!is_dir($outputDir)) {
            mkdir($outputDir);
        }

        // create directory for help files for backward compatibility
        $helpFilesDir = "$this->_productBaseDir/admin/htdocs/locales/$this->_localeCode/";
        if (!is_dir($helpFilesDir)) {
            mkdir($helpFilesDir);
        }

        copy("{$inputDir}common_messages.php", "{$outputDir}common_messages_{$this->_localeCode}.php");
        copy("{$inputDir}conhelp.php", "{$outputDir}conhelp_{$this->_localeCode}.php");
        copy("{$inputDir}messages.php", "{$outputDir}messages_{$this->_localeCode}.php");

        if ($this->_isOsWindow()) {
            copy("{$inputDir}cli_messages.php", "{$outputDir}cli_messages_{$this->_localeCode}.php");
        }

        // add locale code to included files names in main locale file
        $mainLocaleFile = file_get_contents("{$outputDir}messages_{$this->_localeCode}.php");
        $mainLocaleFile = preg_replace('/(@include_once\(\'[a-z_]+)(\.php)/m', "\\1_{$this->_localeCode}\\2", $mainLocaleFile);
        file_put_contents("{$outputDir}messages_{$this->_localeCode}.php", $mainLocaleFile);

        if ($this->_isOsUnix()) {
            $sqlAddLocaleCode = "replace into locales (id) values ('$this->_localeCode')";
            $this->_exec("$this->_mysqlDir/mysql -uadmin -p`cat /etc/psa/.psa.shadow` psa -e \"$sqlAddLocaleCode\"");
        } else {
            $this->_exec("\"$this->_productBaseDir/admin/bin/LP.exe\" -ADD \"-ID=$this->_localeCode\" \"-NAME=$this->_localeCode\"");
        }
    }

    /**
     * Import SiteEditor locale
     *
     */
    private function _importSiteEditorLocale()
    {
        $seLocaleCode = str_replace('-', '_', $this->_localeCode);

        $localeFile = "$this->_inputDir/siteeditor/locale.xml";

        $outputDir = "$this->_productSiteEditorDir/resources/locale/$seLocaleCode";
        if (!is_dir($outputDir)) {
            mkdir($outputDir);
        }

        $infoXmlFile = "$outputDir/info.xml";
        $converter = dirname(__FILE__) . "/helpers/se_convert_locales.php";
        $this->_exec("$this->_phpBinary \"$converter\" --mode=locale --inputfile=\"$localeFile\" --outdir=\"$outputDir\"");

        if ($this->_isOsUnix()) {
            $this->_exec("/usr/local/bin/se_config --update_locales");
        } else {
            $sePhpIni = "$this->_productSiteEditorDir/etc/php.ini";
            $localesUpdater = "$this->_productSiteEditorDir/utils/update_locales.php";
            $this->_exec("$this->_phpBinary -c \"$sePhpIni\" \"$localesUpdater\" all");
        }
    }

    /**
     * Get XML with language pack description
     *
     * @return SimpleXMLElement
     */
    private function _getLocaleDescriptionXml()
    {
        static $xml;

        if (!$xml) {
            $xml = simplexml_load_file("$this->_inputDir/info.xml");
        }

        return $xml;
    }

}

$application = new Lkit_Application_Import();
$application->run();